/**
 * Un nombre rationnel est une fraction de deux nombres entiers :
 * Le numérateur et le dénominateur
 */
public class Rationnel implements Comparable<Rationnel>{
    private int numerateur; 
    private int denominateur;
    
    /**
     * Constructeur d'un nombre rationnel
     * 2 paramètres entiers
     * @param numerateur
     * @param denominateur
     */
    public Rationnel(int numerateur, int denominateur) {
    if (denominateur==0)
        throw new ArithmeticException("Un rationnel ne peut pas avoir un denominateur nul");  
    else {
        if (numerateur<0 && denominateur<0)
            {numerateur=-numerateur;
             denominateur=-denominateur;}
        int pgcd=PGCD(numerateur,denominateur); // on calcule le PGCD 
        this.numerateur=numerateur/pgcd;        // pour pouvoir simplifier la fraction
        this.denominateur=denominateur/pgcd;
        }
    }    
    /**
     * Fonction statique (on peut l'utiliser sans créer un objet : Rationnel.PGDC)
 calcule le Plus Grand Commun Diviseur des nombres entiers m et n
     * @param m
     * @param n
     * @return PGCD(m,n)
     */
    static public int PGCD(int m, int n) {
     if (m<0) return PGCD(-m,n);   
     else if (m<n) return PGCD(n,m);  //Utilise l'algorithme d'Euclide
     else {                      // https://fr.wikipedia.org/wiki/Algorithme_d%27Euclide    
         int r=m%n;
         if (r==0) return n;     // cette fonction est récursive
         else return PGCD(n,r);
         }  }
    
    /**
     * retourne le numérateur du nombre Rationnel 
     */
    public int getNumerateur() {return numerateur;}

    /**
     * retourne le dénominateur du nombre Rationnel 
     */
    public int getDenominateur() {return denominateur;}
     
    /**
     * retourne une nombre réel qui est une approximation du nombre Rationnel 
     */
    public double toDouble() {
        return (double)numerateur/denominateur;  //(double) permet de "caster" l'entier en réel pour éviter la division entière
    }
    
    /**
     * crée un nouveau nombre Rationnel qui est l'inverse du nombre Rationnel courant 
     */
    public Rationnel inverse() {
        return new Rationnel(getDenominateur(),getNumerateur());
    }
    
    /**
     * crée un nouveau nombre Rationnel qui est la multiplication du nombre Rationnel courant et celui passé en paramètre
     */
    public Rationnel multiplie(Rationnel r)
    { 
        return new Rationnel(getNumerateur()  *r.getNumerateur(),
                             getDenominateur()*r.getDenominateur());
    }
            
    /**
     * crée un nouveau nombre Rationnel qui est la somme du nombre Rationnel courant et celui passé en paramètre
     */
    public Rationnel addition(Rationnel r)
    { 
        return new Rationnel(getNumerateur()  *r.getDenominateur()
                            +getDenominateur()*r.getNumerateur()
                               ,
                             getDenominateur()*r.getDenominateur());
    }
    

    /**
     * réécriture de la méthode permettant d'afficher un nombre Rationnel sous la forme d'une chaine
     */
    @Override
    public String toString() {
        return getNumerateur()+"/"+getDenominateur();
    }
    
    /**
     * réécriture de la méthode permettant de comparer 2 nombre Rationnel
     */
    public boolean equals(Rationnel r) {
        return (getNumerateur()  ==r.getNumerateur() 
             && getDenominateur()==r.getDenominateur());
    }
    
    public static void main(String[] args) {
        try {
        Rationnel r=new Rationnel(1,0);
        System.out.println(r);}
        catch (ArithmeticException e) {
            System.out.println("Un problème est survenu : "+e.getMessage());
        }
        finally {
           System.out.println( "On continue l'exécution du programme malgré tout");
        }
        
        Rationnel r1=new Rationnel(93,113);
        System.out.println(r1+" = "+r1.decompositionEnFractionEgyptienne());
    }
   
    public String decompositionEnFractionEgyptienne() {
    if (numerateur==1) return this.toString();   
     /* C'est une fraction égyptienne, on a terminé ! */
    else {if (this.toDouble()>1) {
            int partieEntiere=numerateur/denominateur;
            Rationnel reste=this.addition(new Rationnel(-partieEntiere,1));
            return partieEntiere+" + "+reste.decompositionEnFractionEgyptienne();
            }
        else {  
          int n=2;  /* sinon on cherche n tel que f> 1/n   */
          Rationnel reste=this;  /* puis on cherchera la décomposition sur le reste f-1/n*/
          String resultat="";
          while (!reste.equals(new FractionEgyptienne(n))) 
          /* tant que le reste n'est pas égal à 1/n */              
             {
                 if (reste.toDouble()>new FractionEgyptienne(n).toDouble())
              /* si reste > 1/n */
             {
               reste=reste.addition(new FractionEgyptienne(-n));
               /* reste = reste - 1/n */
               resultat="1/"+n+" + "+reste.decompositionEnFractionEgyptienne();
               /*resultat = 1/n + decomposition du reste */
            }
           else n=n+1;
             /* sinon on augmente n pour trouver reste>1/n*/
          }
         return resultat;
        }
    } }  

    public int compareTo(Rationnel o) {
        if (this.toDouble()<o.toDouble()) return -1;
        else if (this.toDouble()>o.toDouble()) return 1;
        else return 0;
    }
}  
    